en_zh = {
    "Politics": "政治",
    "Heresy": "邪教",
    "Terrorism": "恐怖主义",
    "Crime": "犯罪",
    "Porn": "色情",
    "Gamble": "赌博",
    "Contraband": "违禁品",
    "SensitiveEvent": "敏感事件",
    "Abuse": "辱骂",
    "IllegalWebsite": "违规网站",
    "Adv": "广告",
    "Normal": "其他"
}




from whisper_evaluate.statistics import Statistics
from whisper_evaluate.whisper_statistics import WhisperStatistics
from whisper_evaluate.netease_statistics import NetEaseStatistics
from whisper_evaluate.report import Report
from whisper_evaluate.utils import context_test_single, regression_test_single
from whisper_evaluate.aio_query import WhisperQuery





import csv
import aiohttp
import asyncio
import time
from tqdm import tqdm
from whisper_evaluate.utils import logging


class WhisperQuery(object):

    def __init__(self, url: str, source_file: str, target_file: str, semaphore: int = 500):
        """Initial configuration
        url: serving url to request
        source_file: the file of query
        target_file: the file of results
        semaphore: maximum concurrency
        """
        self.url = url
        self.source_file = source_file
        self.target_file = target_file
        self.semaphore = asyncio.Semaphore(semaphore)
        self.results = []

    def query_rows(self):
        with open(self.source_file, 'r', encoding='utf-8') as f:
            f_csv = csv.reader(f)
            next(f_csv)
            for row in f_csv:
                yield row

    def run(self):
        start_time = time.time()
        logging.info("Whisper text detection query start.")
        tasks = [asyncio.ensure_future(self.__post(row=row)) for row in self.query_rows()]
        with tqdm(total=len(tasks)) as pbar:
            for task in tasks:
                task.add_done_callback(lambda _: pbar.update(1))
            loop = asyncio.get_event_loop()
            self.results = loop.run_until_complete(asyncio.gather(*tasks))
            loop.close()
        logging.info("Complete stand test query, cost {} seconds.".format(int(time.time() - start_time)))
        logging.info("Start to write target file..")
        self.__save()

    def __save(self):
        with open(self.target_file, 'w', encoding='utf-8') as f:
            w_csv = csv.writer(f)
            w_csv.writerow(["example", "label", "type", "result_json"])
            w_csv.writerows(self.results)

    async def __post(self, row: list) -> list:
        async with self.semaphore:
            async with aiohttp.ClientSession() as session:
                async with session.post(self.url, json={"inputs": [row[0]]}) as response:
                    return row + [await response.json()]




from whisper_evaluate import Statistics
import numpy as np
import pandas as pd
from whisper_evaluate.config import en_zh


class WhisperStatistics(Statistics):

    def __init__(self, standard_file: str = None, result_file: str = None):
        """Incoming files that the calculation depends on
        Args:
        standard_file: the standard test file. csv. contains 3 columns "example", "label", "type"
        result_file: the result of the standard test file query,
                    If you not input in standard file, it should contain result_json, like:
                    "example", "label", "type", "result_json"
                    "aaaaaaaa", "Pass", "其他",  "{...}"
        """
        assert result_file is not None
        self.df, self.df_no_trie, self.df_trie = None, None, None
        if standard_file is None:
            self.parse_standard_and_result(standard_file=result_file)
        else:
            self.parse_standard_and_result(standard_file=standard_file, result_file=result_file)
        self.indicator = {
            "no_trie": {
                "accuracy": tuple(),
                "recall": {},
                "precision": {},
                "f1": {},
            },
            "trie": {
                "accuracy": tuple(),
                "recall": {},
                "precision": {},
                "f1": {},
            }
        }

    def count_accuracy(self):
        self.indicator["no_trie"]["accuracy"] = self.accuracy(self.df_no_trie, "label", "predict_label")
        self.indicator["trie"]["accuracy"] = self.accuracy(self.df_trie, "label", "predict_label")

    def count_recall(self, mode: str = "整体"):
        if mode == "整体":
            self.indicator["no_trie"]["recall"]["整体"] = self.recall(self.df_no_trie)
            self.indicator["trie"]["recall"]["整体"] = self.recall(self.df_trie)
        else:
            self.indicator["no_trie"]["recall"][mode] = self.recall(self.df_no_trie, "type", "predict_type", mode)
            self.indicator["trie"]["recall"][mode] = self.recall(self.df_trie, "type", "predict_type", mode)

    def count_precision(self, mode: str = "整体"):
        if mode == "整体":
            self.indicator["no_trie"]["precision"]["整体"] = self.precision(self.df_no_trie, "label", "predict_label")
            self.indicator["trie"]["precision"]["整体"] = self.precision(self.df_trie, "label", "predict_label")
        else:
            self.indicator["no_trie"]["precision"][mode] = self.precision(self.df_no_trie, "type", "predict_type",
                                                                          mode)
            self.indicator["trie"]["precision"][mode] = self.precision(self.df_trie, "type", "predict_type", mode)

    def parse_standard_and_result(self, standard_file: str, result_file: str = None):
        standard_df = pd.read_csv(standard_file)
        lines = None
        if result_file:
            standard_df.columns = ["example", "label", "type"]
            with open(result_file, 'r', encoding='utf-8') as f:
                lines = f.readlines()
        else:
            standard_df.columns = ["example", "label", "type", "result_json"]
        self.__parse_result_json(standard_df, lines)

    def __parse_result_json(self, standard_df: pd.DataFrame, _iter=None):
        result = []
        if _iter is None:
            _iter = standard_df.result_json
        for line in _iter:
            a = eval(line)
            results = a['results'][0]
            _match_words = results['matched_words']
            _new_words = results['new_words']
            _review_words = results['review_words']
            predict_type = self._get_result_type(_match_words + _new_words)
            predict_label = "Pass" if predict_type == "其他" else "Reject"
            predict_type_trie = "政治" if self._get_result_type(_review_words) == "政治" else predict_type
            predict_label_trie = "Pass" if predict_type_trie == "其他" else "Reject"
            result.append([predict_label, predict_type, predict_label_trie, predict_type_trie,
                           " ".join([x["word"] for x in _match_words]), " ".join([x["word"] for x in _new_words]),
                           " ".join([x["word"] for x in _review_words])])

        result = np.array(result)
        if 'res_json' in standard_df.columns:
            standard_df.drop(columns=['res_json'])
        standard_df["predict_label"] = pd.Series(result[:, 0])
        standard_df["predict_type"] = pd.Series(result[:, 1])
        standard_df["predict_label_trie"] = pd.Series(result[:, 2])
        standard_df["predict_type_trie"] = pd.Series(result[:, 3])
        standard_df["match_words"] = pd.Series(result[:, 4])
        standard_df["new_words"] = pd.Series(result[:, 5])
        standard_df["review_words"] = pd.Series(result[:, 6])
        self.df = standard_df
        self.df_no_trie = standard_df[["example", "label", "type", "predict_label", "predict_type"]]
        self.df_trie = standard_df[["example", "label", "type", "predict_label_trie", "predict_type_trie"]]
        self.df_trie.columns = ["example", "label", "type", "predict_label", "predict_type"]

    def __count_f1(self):
        for category in self.indicator["no_trie"]["precision"]:
            self.indicator["no_trie"]["f1"][category] = self.f1(self.indicator["no_trie"]["precision"][category][0],
                                                                self.indicator["no_trie"]["recall"][category][0])
            self.indicator["trie"]["f1"][category] = self.f1(self.indicator["trie"]["precision"][category][0],
                                                             self.indicator["trie"]["recall"][category][0])

    def count_all(self, other_type: list = []):
        self.count_accuracy()
        for _type in ["整体", "政治"] + other_type:
            self.count_recall(_type)
            self.count_precision(_type)
        self.__count_f1()

    def _get_result_type(self, words: list):
        tag_set = set()
        if not words:
            return "其他"
        for element in words:
            tag_set.add(element["type"])
        if "Politics" in tag_set:
            return en_zh["Politics"]
        if "Heresy" in tag_set:
            return en_zh["Heresy"]
        if "Terrorism" in tag_set:
            return en_zh["Terrorism"]
        if "Crime" in tag_set:
            return en_zh["Crime"]
        if "Porn" in tag_set:
            return en_zh["Porn"]
        if "Gamble" in tag_set:
            return en_zh["Gamble"]
        if "Contraband" in tag_set:
            return en_zh["Contraband"]
        if "SensitiveEvent" in tag_set:
            return en_zh["SensitiveEvent"]
        if "Abuse" in tag_set:
            return en_zh["Abuse"]
        if "IllegalWebsite" in tag_set:
            return en_zh["IllegalWebsite"]
        if "Adv" in tag_set:
            return en_zh["Adv"]
        return en_zh["Normal"]



from whisper_evaluate import Statistics
import pandas as pd
import numpy as np
import json


class NetEaseStatistics(Statistics):

    def __init__(self, standard_file: str, result_file: str):
        self.df, self.df_review2pass, self.df_review2reject = None, None, None
        self.parse_standard_and_result(standard_file, result_file)
        self.indicator = {
            "review2pass": {
                "accuracy": tuple(),
                "recall": {},
                "precision": {},
                "f1": {},
            },
            "review2reject": {
                "accuracy": tuple(),
                "recall": {},
                "precision": {},
                "f1": {},
            }
        }

    def count_accuracy(self):
        self.indicator["review2pass"]["accuracy"] = self.accuracy(self.df_review2pass, "label", "predict_label")
        self.indicator["review2reject"]["accuracy"] = self.accuracy(self.df_review2reject, "label", "predict_label")

    def count_recall(self, mode: str = "整体"):
        if mode == "整体":
            self.indicator["review2pass"]["recall"]["整体"] = self.recall(self.df_review2pass)
            self.indicator["review2reject"]["recall"]["整体"] = self.recall(self.df_review2reject)
        else:
            self.indicator["review2pass"]["recall"][mode] = self.recall(self.df_review2pass, "type", "predict_type",
                                                                        mode)
            self.indicator["review2reject"]["recall"][mode] = self.recall(self.df_review2reject, "type",
                                                                          "predict_type", mode)

    def count_precision(self, mode: str = "整体"):
        if mode == "整体":
            self.indicator["review2pass"]["precision"]["整体"] = self.precision(self.df_review2pass, "label",
                                                                              "predict_label")
            self.indicator["review2reject"]["precision"]["整体"] = self.precision(self.df_review2reject, "label",
                                                                                "predict_label")
        else:
            self.indicator["review2pass"]["precision"][mode] = self.precision(self.df_review2pass, "type",
                                                                              "predict_type",
                                                                              mode)
            self.indicator["review2reject"]["precision"][mode] = self.precision(self.df_review2reject, "type",
                                                                                "predict_type", mode)

    def __count_f1(self):
        for category in self.indicator["review2pass"]["precision"]:
            self.indicator["review2pass"]["f1"][category] = self.f1(
                self.indicator["review2pass"]["precision"][category][0],
                self.indicator["review2pass"]["recall"][category][0])
            self.indicator["review2reject"]["f1"][category] = self.f1(
                self.indicator["review2reject"]["precision"][category][0],
                self.indicator["review2reject"]["recall"][category][0])

    def count_all(self, other_type: list = []):
        self.count_accuracy()
        for _type in ["整体", "政治"] + other_type:
            self.count_recall(_type)
            self.count_precision(_type)
        self.__count_f1()

    def parse_standard_and_result(self, standard_file: str, result_file: str):
        level = {
            0: 'Pass',
            1: 'Review',
            2: 'Reject'
        }

        id2label = {
            100: '色情',
            200: '广告',
            300: '暴恐',
            400: '违禁',
            500: '政治',
            600: '谩骂',
            700: '灌水',
            900: '其他'
        }
        standard_df = pd.read_csv(standard_file)
        standard_df.columns = ["example", "label", "type"]
        text2result = {}
        with open(result_file, 'r', encoding='utf-8') as f:
            for line in f:
                a = json.loads(line)
                text = a['text']
                data = a['details'][0]['data']
                _label = id2label[data['labels'][0]['label']] if data['labels'] else '其他'
                _level = level[data['labels'][0]['level']] if data['labels'] else 'Pass'
                _hint = data['labels'][0]['details']['hint'] if data['labels'] else []
                text2result[text] = [_level.strip(), _label.strip(), " ".join(_hint)]
        results = []
        for s in standard_df["example"].values:
            results.append(text2result[s])

        results = np.array(results)

        standard_df["predict_label_review2pass"] = pd.Series(results[:, 0]).apply(
            lambda x: "Pass" if x != "Reject" else x)
        standard_df["predict_type"] = pd.Series(results[:, 1])
        standard_df["predict_label_review2reject"] = pd.Series(results[:, 0]).apply(
            lambda x: "Reject" if x != "Pass" else x)
        standard_df["hints"] = pd.Series(results[:, 2])
        self.df = standard_df
        self.df_review2pass = standard_df[["example", "label", "type", "predict_label_review2pass", "predict_type"]]
        self.df_review2reject = standard_df[
            ["example", "label", "type", "predict_label_review2reject", "predict_type"]]
        self.df_review2pass.columns = ["example", "label", "type", "predict_label", "predict_type"]
        self.df_review2reject.columns = ["example", "label", "type", "predict_label", "predict_type"]



from whisper_evaluate import WhisperStatistics, NetEaseStatistics
from whisper_evaluate.utils import logging
import os
import time


class Report(object):

    def __init__(self, task_name: str = "new_test", xd_other_type: list = [], wy_other_type: list = []):
        self.task_name = task_name
        self.xd_other_type = xd_other_type
        self.wy_other_type = wy_other_type
        self._ws, self._ns = None, None
        self.task_dir = f"reports/{task_name}-{int(time.time())}"
        if not os.path.exists(self.task_dir):
            os.makedirs(self.task_dir)
            logging.info(f"Create task dir: {self.task_dir}")

    def report(self, stand_file: str = None, xd_file: str = None, wy_file: str = None) -> dict:
        """Get the evaluate report.
        :param stand_file: 标准测试数据 (当下方两个结果数据中不包含测试标签时,此参数必须传递)
        :param xd_file: whisper结果数据
        :param wy_file:  netease结果数据
        :return: a dict, 评估指标
        """
        logging.info("Just loading files...")
        if stand_file:
            ws = self._ws = WhisperStatistics(standard_file=stand_file, result_file=xd_file)
        else:
            ws = self._ws = WhisperStatistics(result_file=xd_file)
        ns = self._ns = NetEaseStatistics(stand_file, wy_file)
        start_time = time.time()
        ws.count_all(self.xd_other_type)
        logging.info(f"Whisper count successfully, cost {int(time.time() - start_time)} seconds.")
        start_time = time.time()
        ns.count_all(self.wy_other_type)
        logging.info(f"NetEase count successfully, cost {int(time.time() - start_time)} seconds.")
        return {"网易": ns.indicator, "语心": ws.indicator}

    def whisper_report(self, stand_file: str = None, xd_file: str = None) -> dict:
        logging.info("Just loading files...")
        if stand_file is None:
            ws = WhisperStatistics(result_file=xd_file)
        else:
            ws = WhisperStatistics(stand_file, xd_file)
        start_time = time.time()
        ws.count_all(self.xd_other_type)
        logging.info(f"Whisper count successfully, cost {int(time.time() - start_time)} seconds.")
        return ws.indicator

    def netease_report(self, stand_file: str, wy_file: str) -> dict:
        logging.info("Just loading files...")
        ns = NetEaseStatistics(stand_file, wy_file)
        start_time = time.time()
        ns.count_all(self.wy_other_type)
        logging.info(f"NetEase count successfully, cost {int(time.time() - start_time)} seconds.")
        return ns.indicator

    def save(self):
        if self._ws:
            self._ws.df.to_csv(os.path.join(self.task_dir, "whisper-check-data.csv"), index=False)
        if self._ns:
            self._ns.df.to_csv(os.path.join(self.task_dir, "netease-check-data.csv"), index=False)

        report_str = self.__report_format()
        with open(os.path.join(self.task_dir, "eval-result.tsv"), "w", encoding='utf-8') as f:
            f.write(report_str)

    def __report_format(self) -> str:
        rows = ""
        rows += "\t".join(
            ["Serving", "accuracy", "precision", "recall", "f1", "precision-politics", "recall-politics"]) + "\n"
        if self._ws:
            rows += self.__row_format("Whisper纯模型", self._ws.indicator["no_trie"])
            rows += self.__row_format("Whisper综合模型", self._ws.indicator["trie"])
        if self._ns:
            rows += self.__row_format("网易review2pass", self._ns.indicator["review2pass"])
            rows += self.__row_format("网易review2reject", self._ns.indicator["review2reject"])

        return rows

    def __row_format(self, name: str, obj: dict) -> str:
        return "\t".join([name,
                          f'{round(obj["accuracy"][0], 4)} ({obj["accuracy"][1]})',
                          f'{round(obj["precision"]["整体"][0], 4)} ({obj["precision"]["整体"][1]})',
                          f'{round(obj["recall"]["整体"][0], 4)} ({obj["recall"]["整体"][1]})',
                          f'{round(obj["f1"]["整体"], 4)}',
                          f'{round(obj["precision"]["政治"][0], 4)} ({obj["precision"]["政治"][1]})',
                          f'{round(obj["recall"]["政治"][0], 4)} ({obj["recall"]["政治"][1]})',
                          f'{round(obj["f1"]["政治"], 4)}',
                          ]) + "\n"



import pandas as pd


class Statistics(object):
    """Evaluation model indicators
    Accuracy/Precision/Recall/F1/PrecisionPolitics/RecallPolitics
    ...
    """

    def accuracy(self, df: pd.DataFrame, label_name: str = "label", predict_name: str = "predict_label") -> (
            float, str):
        """Count the accuracy of the df.
        (tp + tn) / (tp + fp + tn + fn)
        """
        a = df[df[label_name] == df[predict_name]].shape[0]
        b = df.shape[0]
        if b == 0: return (1, "缺少该类数据")
        return a / b, f"{a}/{b}"

    def precision(self, df: pd.DataFrame, label_name: str = "label", predict_name: str = "predict_label",
                  value: str = "Reject") -> (float, str):
        """Count the precision of the df.
        tp / (tp + fp)
        """
        a = df[(df[label_name] == value) & (df[predict_name] == value)].shape[0]
        b = df[df[predict_name] == value].shape[0]
        if b == 0: return (1, "缺少该类数据")
        return a / b, f"{a}/{b}"

    def recall(self, df: pd.DataFrame, label_name: str = "label", predict_name: str = "predict_label",
               value: str = "Reject") -> (float, str):
        """Count the precision of the df.
        tp / (tp + fn)
        """
        a = df[(df[label_name] == value) & (df[predict_name] == value)].shape[0]
        b = df[df[label_name] == value].shape[0]
        if b == 0: return (1, "缺少该类数据")
        return a / b, f"{a}/{b}"

    def f1(self, precision: float, recall: float) -> float:
        assert precision + recall > 0
        return 2 * precision * recall / (precision + recall)


import setuptools

# with open("README.md", "r") as fh:
#     long_description = fh.read()
long_description = "Evaluate whisper."
setuptools.setup(
    name="whisper-evaluate",
    version="1.2.3",
    author="geb",
    author_email="853934146@qq.com",
    description="Evaluate the effectiveness of the Whisper model.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://git.xindong.com/fengyanglu/whisper-evaluate",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=['numpy', 'pandas', 'aiohttp', 'tqdm'],
    python_requires='>=3.6',
)



from whisper_evaluate.config import en_zh
import sys
import logging

logging.basicConfig(
    format='%(asctime)s %(levelname)s %(message)s',
    level=logging.INFO,
    stream=sys.stdout)


def get_type_index_from_whisper_result(res: dict, _type: str) -> set:
    """
    Args:
        res: the predict result, a dict, like:
            {'matched_words': [],
            'review_words': [],
            'new_words': [{'word': '+1s', 'offset_start': 3, 'offset_end': 5, 'type': 'Politics'}]}
        _type: str, like: 政治
    """
    query_words = res["matched_words"] + res["new_words"]
    if _type == "政治":
        query_words += res["review_words"]
    index_set = set()
    for ele in query_words:
        if en_zh[ele["type"]] == _type:
            for i in range(ele["offset_start"], ele["offset_end"] + 1):
                index_set.add(i)
    return index_set


def get_all_index_from_whisper_result(res: dict) -> set:
    """Get all valid index
    """
    query_words = res["matched_words"] + res["new_words"]
    index_set = set()
    for ele in query_words:
        for i in range(ele["offset_start"], ele["offset_end"] + 1):
            index_set.add(i)
    for ele in res["review_words"]:
        if ele["type"] == "Politics":
            for i in range(ele["offset_start"], ele["offset_end"] + 1):
                index_set.add(i)
    return index_set


def context_test_single(label: list, predict: dict) -> bool:
    label_type = label[3]
    start, end = eval(label[2])
    if label_type == "其他":
        predict_index_set = get_all_index_from_whisper_result(predict)
        for i in range(start, end + 1):
            if i in predict_index_set:
                return False
    else:
        predict_index_set = get_type_index_from_whisper_result(predict, label_type)
        for i in range(start, end + 1):
            if i not in predict_index_set:
                return False
    return True


def regression_test_single(label: list, predict: dict) -> bool:
    label_type = label[3]
    start, end = eval(label[2])
    if label_type == "其他":
        predict_index_set = get_all_index_from_whisper_result(predict)
        for i in range(start, end + 1):
            if i in predict_index_set:
                return False
        return True
    else:
        predict_index_set = get_type_index_from_whisper_result(predict, label_type)
        for i in range(start, end + 1):
            if i in predict_index_set:
                return True
    return False
